@@ -1,4 +1,4 @@
-""" Least Squares Methodos for parameter estimation """
+""" Least Squares Methods for parameter estimation """
 
 # Authors:
 #           Wilson Rocha Lacerda Junior <wilsonrljr@outlook.com>
@@ -7,10 +7,11 @@
 
 import numpy as np
 import warnings
+from ..narmax_base import InformationMatrix
 
 
-class Estimators:
-    """Oridanry Least squares for linear parameter estimation"""
+class Estimators(InformationMatrix):
+    """Ordinary Least Squares for linear parameter estimation"""
 
     def __init__(
         self,
@@ -96,25 +97,79 @@ class Estimators:
 
         References
         ----------
-        [1] Manuscript: Sorenson, H. W. (1970). Least-squares estimation:
-            from Gauss to Kalman. IEEE spectrum, 7(7), 63-68.
-            http://pzs.dstu.dp.ua/DataMining/mls/bibl/Gauss2Kalman.pdf
-        [2] Book (Portuguese): Aguirre, L. A. (2007). Introduçaoa identificaçao
-            de sistemas: técnicas lineares enao-lineares aplicadas a sistemas
-            reais. Editora da UFMG. 3a ediçao.
-        [3] Manuscript: Markovsky, I., & Van Huffel, S. (2007).
-            Overview of total least-squares methods.
-            Signal processing, 87(10), 2283-2302.
-            https://eprints.soton.ac.uk/263855/1/tls_overview.pdf
-        [4] Wikipedia entry on Least Squares
-            https://en.wikipedia.org/wiki/Least_squares
+        .. [1] Manuscript: Sorenson, H. W. (1970). Least-squares estimation:
+           from Gauss to Kalman. IEEE spectrum, 7(7), 63-68.
+           http://pzs.dstu.dp.ua/DataMining/mls/bibl/Gauss2Kalman.pdf
+        .. [2] Book (Portuguese): Aguirre, L. A. (2007). Introduçaoa identificação
+           de sistemas: técnicas lineares e não-lineares aplicadas a sistemas
+           reais. Editora da UFMG. 3a edição.
+        .. [3] Manuscript: Markovsky, I., & Van Huffel, S. (2007).
+           Overview of total least-squares methods.
+           Signal processing, 87(10), 2283-2302.
+           https://eprints.soton.ac.uk/263855/1/tls_overview.pdf
+        .. [4] Wikipedia entry on Least Squares
+           https://en.wikipedia.org/wiki/Least_squares
 
         """
         self._check_linear_dependence_rows(psi)
 
         y = y[self.max_lag :, 0].reshape(-1, 1)
-        theta = np.dot(np.linalg.pinv(np.dot(psi.T, psi)), np.dot(psi.T, y))
+        # theta = np.dot(np.linalg.pinv(np.dot(psi.T, psi)), np.dot(psi.T, y))
+        theta = np.linalg.lstsq(psi, y, rcond=None)[0]
         return theta
+    
+    def _unbiased_estimator(self, psi, y, theta, non_degree, elag, max_lag):
+        """Estimate the model parameters using Extended Least Squares method.
+
+        Parameters
+        ----------
+        psi : ndarray of floats
+            The information matrix of the model.
+        X : ndarray of floats
+            The input data to be used in the training process.
+        y_train : array-like of shape = y_training
+            The data used to training the model.
+        biased_theta : array-like of shape = number_of_model_elements
+            The estimated biased parameters of the model.
+
+        Returns
+        -------
+        theta : array-like of shape = number_of_model_elements
+            The estimated unbiased parameters of the model.
+
+        References
+        ----------
+        .. [1] Manuscript: Sorenson, H. W. (1970). Least-squares estimation:
+           from Gauss to Kalman. IEEE spectrum, 7(7), 63-68.
+           http://pzs.dstu.dp.ua/DataMining/mls/bibl/Gauss2Kalman.pdf
+        .. [2] Book (Portuguese): Aguirre, L. A. (2007). Introdução a identificação
+           de sistemas: técnicas lineares e não-lineares aplicadas a sistemas
+           reais. Editora da UFMG. 3a edição.
+        .. [3] Manuscript: Markovsky, I., & Van Huffel, S. (2007).
+           Overview of total least-squares methods.
+           Signal processing, 87(10), 2283-2302.
+            https://eprints.soton.ac.uk/263855/1/tls_overview.pdf
+        .. [4] Wikipedia entry on Least Squares
+           https://en.wikipedia.org/wiki/Least_squares
+
+        """
+        e = y[max_lag :, 0].reshape(-1, 1) - np.dot(psi, theta)
+        for i in range(30):
+            e = np.concatenate([np.zeros([max_lag, 1]), e], axis=0)
+            
+            lagged_data = self.build_output_matrix(e, elag)
+        
+            e_regressors = self.basis_function.fit(
+                lagged_data, max_lag, predefined_regressors=None
+            )
+            
+            psi_extended = np.concatenate([psi, e_regressors], axis=1)
+            unbiased_theta = getattr(self, self.estimator)(psi_extended, y)
+            e = y[max_lag :, 0].reshape(-1, 1) - np.dot(
+                psi_extended, unbiased_theta.reshape(-1, 1)
+            )
+
+        return unbiased_theta[0 : theta.shape[0], 0].reshape(-1, 1)
 
     def total_least_squares(self, psi, y):
         """Estimate the model parameters using Total Least Squares method.
@@ -133,15 +188,15 @@ class Estimators:
 
         References
         ----------
-        [1] Manuscript: Golub, G. H., & Van Loan, C. F. (1980).
-            An analysis of the total least squares problem.
-            SIAM journal on numerical analysis, 17(6), 883-893.
-        [2] Manuscript: Markovsky, I., & Van Huffel, S. (2007).
-            Overview of total least-squares methods.
-            Signal processing, 87(10), 2283-2302.
-            https://eprints.soton.ac.uk/263855/1/tls_overview.pdf
-        [3] Wikipedia entry on Total Least Squares
-            https://en.wikipedia.org/wiki/Total_least_squares
+        .. [1] Manuscript: Golub, G. H., & Van Loan, C. F. (1980).
+           An analysis of the total least squares problem.
+           SIAM journal on numerical analysis, 17(6), 883-893.
+        .. [2] Manuscript: Markovsky, I., & Van Huffel, S. (2007).
+           Overview of total least-squares methods.
+           Signal processing, 87(10), 2283-2302.
+           https://eprints.soton.ac.uk/263855/1/tls_overview.pdf
+        .. [3] Wikipedia entry on Total Least Squares
+           https://en.wikipedia.org/wiki/Total_least_squares
 
         """
         y = y[self.max_lag :, 0].reshape(-1, 1)
@@ -169,7 +224,7 @@ class Estimators:
             The information matrix of the model.
         y_train : array-like of shape = y_training
             The data used to training the model.
-
+        
         Returns
         -------
         theta : array-like of shape = number_of_model_elements
@@ -183,9 +238,9 @@ class Estimators:
 
         References
         ----------
-        [1] Book (Portuguese): Aguirre, L. A. (2007). Introduçaoa identificaçao
-            de sistemas: técnicas lineares enao-lineares aplicadas a sistemas
-            reais. Editora da UFMG. 3a ediçao.
+        .. [1] Book (Portuguese): Aguirre, L. A. (2007). Introduçaoa identificação
+           de sistemas: técnicas lineares e não-lineares aplicadas a sistemas
+           reais. Editora da UFMG. 3a edição.
 
         """
         y, n_theta, n, theta, self.xi = self._initial_values(y, psi)
@@ -235,8 +290,8 @@ class Estimators:
 
         References
         ----------
-        [1] Book: Poularikas, A. D. (2017). Adaptive filtering: Fundamentals
-            of least mean squares with MATLAB®. CRC Press.
+        .. [1] Book: Poularikas, A. D. (2017). Adaptive filtering: Fundamentals
+           of least mean squares with MATLAB®. CRC Press.
 
         """
         y, n_theta, n, theta, self.xi = self._initial_values(y, psi)
@@ -277,13 +332,13 @@ class Estimators:
 
         References
         ----------
-        [1] Book: Haykin, S., & Widrow, B. (Eds.). (2003). Least-mean-square
-            adaptive filters (Vol. 31). John Wiley & Sons.
-        [2] Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
-            análise estatística e novas estratégias de algoritmos LMS de passo
-            variável.
-        [3] Wikipedia entry on Least Mean Squares
-            https://en.wikipedia.org/wiki/Least_mean_squares_filter
+        .. [1] Book: Haykin, S., & Widrow, B. (Eds.). (2003). Least-mean-square
+           adaptive filters (Vol. 31). John Wiley & Sons.
+        .. [2] Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
+           análise estatística e novas estratégias de algoritmos LMS de passo
+           variável.
+        .. [3] Wikipedia entry on Least Mean Squares
+           https://en.wikipedia.org/wiki/Least_mean_squares_filter
 
         """
         y, n_theta, n, theta, self.xi = self._initial_values(y, psi)
@@ -323,13 +378,13 @@ class Estimators:
 
         References
         ----------
-        [1]`Book: Hayes, M. H. (2009). Statistical digital signal processing
-            and modeling. John Wiley & Sons.
-        [2]`Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
-            análise estatística e novas estratégias de algoritmos LMS de passo
-            variável.
-        [3] Wikipedia entry on Least Mean Squares
-            https://en.wikipedia.org/wiki/Least_mean_squares_filter
+        .. [1] `Book: Hayes, M. H. (2009). Statistical digital signal processing
+           and modeling. John Wiley & Sons.
+        .. [2] `Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
+           análise estatística e novas estratégias de algoritmos LMS de passo
+           variável.
+        .. [3] Wikipedia entry on Least Mean Squares
+           https://en.wikipedia.org/wiki/Least_mean_squares_filter
 
         """
         y, n_theta, n, theta, self.xi = self._initial_values(y, psi)
@@ -370,13 +425,13 @@ class Estimators:
 
         References
         ----------
-        [1]`Book: Hayes, M. H. (2009). Statistical digital signal processing
-            and modeling. John Wiley & Sons.
-        [2] Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
-            análise estatística e novas estratégias de algoritmos LMS de passo
-            variável.
-        [3] Wikipedia entry on Least Mean Squares
-            https://en.wikipedia.org/wiki/Least_mean_squares_filter
+        .. [1] `Book: Hayes, M. H. (2009). Statistical digital signal processing
+           and modeling. John Wiley & Sons.
+        .. [2] Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
+           análise estatística e novas estratégias de algoritmos LMS de passo
+           variável.
+        .. [3] Wikipedia entry on Least Mean Squares
+           https://en.wikipedia.org/wiki/Least_mean_squares_filter
 
         """
         y, n_theta, n, theta, self.xi = self._initial_values(y, psi)
@@ -421,13 +476,13 @@ class Estimators:
 
         References
         ----------
-        [1] Book: Hayes, M. H. (2009). Statistical digital signal processing
-            and modeling. John Wiley & Sons.
-        [2] Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
-            análise estatística e novas estratégias de algoritmos LMS de passo
-            variável.
-        [3] Wikipedia entry on Least Mean Squares
-            https://en.wikipedia.org/wiki/Least_mean_squares_filter
+        .. [1] Book: Hayes, M. H. (2009). Statistical digital signal processing
+           and modeling. John Wiley & Sons.
+        .. [2] Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
+           análise estatística e novas estratégias de algoritmos LMS de passo
+           variável.
+        .. [3] Wikipedia entry on Least Mean Squares
+           https://en.wikipedia.org/wiki/Least_mean_squares_filter
 
         """
         y, n_theta, n, theta, self.xi = self._initial_values(y, psi)
@@ -471,13 +526,13 @@ class Estimators:
 
         References
         ----------
-        [1] Book: Hayes, M. H. (2009). Statistical digital signal processing
-            and modeling. John Wiley & Sons.
-        [2] Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
-            análise estatística e novas estratégias de algoritmos LMS de passo
-            variável.
-        [3] Wikipedia entry on Least Mean Squares
-            https://en.wikipedia.org/wiki/Least_mean_squares_filter
+        .. [1] Book: Hayes, M. H. (2009). Statistical digital signal processing
+           and modeling. John Wiley & Sons.
+        .. [2] Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
+           análise estatística e novas estratégias de algoritmos LMS de passo
+           variável.
+        .. [3] Wikipedia entry on Least Mean Squares
+           https://en.wikipedia.org/wiki/Least_mean_squares_filter
 
         """
         y, n_theta, n, theta, self.xi = self._initial_values(y, psi)
@@ -519,13 +574,13 @@ class Estimators:
 
         References
         ----------
-        [1] Book: Hayes, M. H. (2009). Statistical digital signal processing
-            and modeling. John Wiley & Sons.
-        [2] Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
-            análise estatística e novas estratégias de algoritmos LMS de passo
-            variável.
-        [3] Wikipedia entry on Least Mean Squares
-            https://en.wikipedia.org/wiki/Least_mean_squares_filter
+        .. [1] Book: Hayes, M. H. (2009). Statistical digital signal processing
+           and modeling. John Wiley & Sons.
+        .. [2] Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
+           análise estatística e novas estratégias de algoritmos LMS de passo
+           variável.
+        .. [3] Wikipedia entry on Least Mean Squares
+           https://en.wikipedia.org/wiki/Least_mean_squares_filter
 
         """
         y, n_theta, n, theta, self.xi = self._initial_values(y, psi)
@@ -569,13 +624,13 @@ class Estimators:
 
         References
         ----------
-        [1] Book: Hayes, M. H. (2009). Statistical digital signal processing
-            and modeling. John Wiley & Sons.
-        [2] Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
-            análise estatística e novas estratégias de algoritmos LMS de passo
-            variável.
-        [3] Wikipedia entry on Least Mean Squares
-            https://en.wikipedia.org/wiki/Least_mean_squares_filter
+        .. [1] Book: Hayes, M. H. (2009). Statistical digital signal processing
+           and modeling. John Wiley & Sons.
+        .. [2] Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
+           análise estatística e novas estratégias de algoritmos LMS de passo
+           variável.
+        .. [3] Wikipedia entry on Least Mean Squares
+           https://en.wikipedia.org/wiki/Least_mean_squares_filter
 
         """
         y, n_theta, n, theta, self.xi = self._initial_values(y, psi)
@@ -618,13 +673,13 @@ class Estimators:
 
         References
         ----------
-        [1] Book: Hayes, M. H. (2009). Statistical digital signal processing
-            and modeling. John Wiley & Sons.
-        [2] Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
-            análise estatística e novas estratégias de algoritmos LMS de passo
-            variável.
-        [3] Wikipedia entry on Least Mean Squares
-            https://en.wikipedia.org/wiki/Least_mean_squares_filter
+        .. [1] Book: Hayes, M. H. (2009). Statistical digital signal processing
+           and modeling. John Wiley & Sons.
+        .. [2] Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
+           análise estatística e novas estratégias de algoritmos LMS de passo
+           variável.
+        .. [3] Wikipedia entry on Least Mean Squares
+           https://en.wikipedia.org/wiki/Least_mean_squares_filter
 
         """
         y, n_theta, n, theta, self.xi = self._initial_values(y, psi)
@@ -668,13 +723,13 @@ class Estimators:
 
         References
         ----------
-        [1] Book: Hayes, M. H. (2009). Statistical digital signal processing
-            and modeling. John Wiley & Sons.
-        [2] Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
-            análise estatística e novas estratégias de algoritmos LMS de passo
-            variável.
-        [3] Wikipedia entry on Least Mean Squares
-            https://en.wikipedia.org/wiki/Least_mean_squares_filter
+        .. [1] Book: Hayes, M. H. (2009). Statistical digital signal processing
+           and modeling. John Wiley & Sons.
+        .. [2] Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
+           análise estatística e novas estratégias de algoritmos LMS de passo
+           variável.
+        .. [3] Wikipedia entry on Least Mean Squares
+           https://en.wikipedia.org/wiki/Least_mean_squares_filter
 
         """
         y, n_theta, n, theta, self.xi = self._initial_values(y, psi)
@@ -718,13 +773,13 @@ class Estimators:
 
         References
         ----------
-        [1] Book: Hayes, M. H. (2009). Statistical digital signal processing
-            and modeling. John Wiley & Sons.
-        [2] Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
-            análise estatística e novas estratégias de algoritmos LMS de passo
-            variável.
-        [3] Wikipedia entry on Least Mean Squares
-            https://en.wikipedia.org/wiki/Least_mean_squares_filter
+        .. [1] Book: Hayes, M. H. (2009). Statistical digital signal processing
+           and modeling. John Wiley & Sons.
+        .. [2] Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
+           análise estatística e novas estratégias de algoritmos LMS de passo
+           variável.
+        .. [3] Wikipedia entry on Least Mean Squares
+           https://en.wikipedia.org/wiki/Least_mean_squares_filter
 
         """
         y, n_theta, n, theta, self.xi = self._initial_values(y, psi)
@@ -765,23 +820,23 @@ class Estimators:
 
         References
         ----------
-        [1] Book: Hayes, M. H. (2009). Statistical digital signal processing
-            and modeling. John Wiley & Sons.
-        [2] Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
-            análise estatística e novas estratégias de algoritmos LMS de passo
-            variável.
-        [3] Manuscript:Gui, G., Mehbodniya, A., & Adachi, F. (2013).
-            Least mean square/fourth algorithm with application to sparse
-            channel estimation. arXiv preprint arXiv:1304.3911.
-            https://arxiv.org/pdf/1304.3911.pdf
-        [4] Manuscript: Nascimento, V. H., & Bermudez, J. C. M. (2005, March).
-            When is the least-mean fourth algorithm mean-square stable?
-            In Proceedings.(ICASSP'05). IEEE International Conference on
-            Acoustics, Speech, and Signal Processing, 2005.
-            (Vol. 4, pp. iv-341). IEEE.
-            http://www.lps.usp.br/vitor/artigos/icassp05.pdf
-        [5] Wikipedia entry on Least Mean Squares
-            https://en.wikipedia.org/wiki/Least_mean_squares_filter
+        .. [1] Book: Hayes, M. H. (2009). Statistical digital signal processing
+           and modeling. John Wiley & Sons.
+        .. [2] Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
+           análise estatística e novas estratégias de algoritmos LMS de passo
+           variável.
+        .. [3] Manuscript:Gui, G., Mehbodniya, A., & Adachi, F. (2013).
+           Least mean square/fourth algorithm with application to sparse
+           channel estimation. arXiv preprint arXiv:1304.3911.
+           https://arxiv.org/pdf/1304.3911.pdf
+        .. [4] Manuscript: Nascimento, V. H., & Bermudez, J. C. M. (2005, March).
+           When is the least-mean fourth algorithm mean-square stable?
+           In Proceedings.(ICASSP'05). IEEE International Conference on
+           Acoustics, Speech, and Signal Processing, 2005.
+           (Vol. 4, pp. iv-341). IEEE.
+           http://www.lps.usp.br/vitor/artigos/icassp05.pdf
+        .. [5] Wikipedia entry on Least Mean Squares
+           https://en.wikipedia.org/wiki/Least_mean_squares_filter
 
         """
         y, n_theta, n, theta, self.xi = self._initial_values(y, psi)
@@ -821,15 +876,15 @@ class Estimators:
 
         References
         ----------
-        [1] Chambers, J. A., Tanrikulu, O., & Constantinides, A. G. (1994).
-            Least mean mixed-norm adaptive filtering.
-            Electronics letters, 30(19), 1574-1575.
-            https://ieeexplore.ieee.org/document/326382
-        [2] Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
-            análise estatística e novas estratégias de algoritmos LMS de passo
-            variável.
-        [3] Wikipedia entry on Least Mean Squares
-            https://en.wikipedia.org/wiki/Least_mean_squares_filter
+        .. [1] Chambers, J. A., Tanrikulu, O., & Constantinides, A. G. (1994).
+           Least mean mixed-norm adaptive filtering.
+           Electronics letters, 30(19), 1574-1575.
+           https://ieeexplore.ieee.org/document/326382
+        .. [2] Dissertation (Portuguese): Zipf, J. G. F. (2011). Classificação,
+           análise estatística e novas estratégias de algoritmos LMS de passo
+           variável.
+        .. [3] Wikipedia entry on Least Mean Squares
+           https://en.wikipedia.org/wiki/Least_mean_squares_filter
 
         """
         y, n_theta, n, theta, self.xi = self._initial_values(y, psi)
