@@ -11,10 +11,13 @@ from .narmax import PolynomialNarmax
 import numpy as np
 from ..residues.residues_correlation import ResiduesAnalysis
 from ..utils._check_arrays import check_X_y
+from ..utils.deprecation import deprecated
 
 
+@deprecated(version='v0.1.7', future_version='v0.2.0',
+            alternative='from sysidentpy.simulation import SimulateNARMAX')
 class SimulatePolynomialNarmax(PolynomialNarmax):
-    """Simulation of Polynomial NARXMAX model
+    """Simulation of Polynomial NARMAX model
 
     Parameters
     ----------
@@ -22,8 +25,8 @@ class SimulatePolynomialNarmax(PolynomialNarmax):
         The number of inputs of the system.
     estimator : str, default="least_squares"
         The parameter estimation method.
-    extended_least_squres : bool, default=False
-        Whether to use extended least squres method
+    extended_least_squares : bool, default=False
+        Whether to use extended least squares method
         for parameter estimation.
         Note that we define a specific set of noise regressors.
     estimate_parameter : bool, default=False
@@ -39,7 +42,7 @@ class SimulatePolynomialNarmax(PolynomialNarmax):
     offset_covariance : float, default=0.2
         The offset covariance factor of the affine least mean squares
         filter.
-    mu : float, defaul=0.01
+    mu : float, default=0.01
         The convergence coefficient (learning rate) of the filter.
     eps : float
         Normalization factor of the normalized filters.
